/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file audio.h
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 25.02.2006
 */

#ifndef AUDIO_H_
#define AUDIO_H_

#include <mad.h>
#include <ao/ao.h>
#include <sys/stat.h>

#include "daisylibao.h"

/**
 * type to identify difrent audio file formats
 */
typedef enum AUDIO_FILE_TYPE 
{
	AUDIO_FILE_WAV		=	1,	/* PCM WAVE file */
	AUDIO_FILE_MP3		=	2,	/* MP3 file. uses mp3.h */
	AUDIO_FILE_AAC		=	3,	/* AAC file. uses aac.h */
	AUDIO_FILE_UNKNOWN	=	-1,	/* Unknown filetype */
} audio_file_type;


/**
 * plays a file from start seconds/fractions to stop second/fractions
 */
int play (const char *fname, char *start, char *stop);


/**
 * This is the data. All information related to the data should be in this struct.
 */
struct audio_data
{
	/* buffer */
	struct buffer
	{
		unsigned char const *buffer_start;
		unsigned long buffer_length;
	} buffer;
	
	char* filename;		/* the filename of the file we have loaded */
	int fd;				/* input file */
	void *fdm;			/* pointer to file mmap */
	struct stat stat;	/* file stats */

	audio_file_type file_type;		/** the file_type (mp3, aac, wav) */
	audio_file_type prev_file_type;	/** the previous file_type */
	
	ao_sample_format format;	/** the format of our output samples. we use this to initialize ao */

	/* mad_timer_t variables to keep track of progress, start and stop-times. */
	mad_timer_t progress;
	mad_timer_t start;
	mad_timer_t stop;
};


#endif /*AUDIO_H_*/
