/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file daisyplayer.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 15.02.2006
 * 
 * Needed to link: -lxml2 -lao -lmad -lfaad
 */

#include <signal.h>
#include <string.h>
#include <unistd.h>

#include "audio.h"
#include "report.h"
#include "control.h"
#include "nccparser.h"
#include "ncxparser.h"
#include "common.h"


// the path to the daisybook
char *path;

// variable used to indicate which standard to parse, if ncx = 0 we shall parse the 2.02 standard (ncc).
int ncx;


/**
 * Listens for CTRL-C and does it's best to free resources in a Good Way
 */
static void sigproc();


/**
 * Start the parsing of the daisy book
 */
static int parse(char* filename);

int main (int argc, char *argv[]) {
	int ret = 0;
	
	// intialising the mp3 and xmlparsing librarys
	//mp3_init();
	parseInit();
	
	/** adding signal handling. this should catch ctrl-c and
	 * free the resources because this is a Good Thing */
	signal(SIGINT, sigproc);
	
	// open a ncc file and start parsing
	if (argc == 2)
	{
		ret = parse(argv[1]);
	}
	else 
	{
		char string[500];
		report ("\nPlease enter full path to the Daisy book (../ncc.html | ../*.ncx)\n", REP_MESSAGE);
		scanf("%s", string);
		if(string != NULL)
		{
			printf ("\nReading from %s\n",string);
			ret = parse(string);
		}			
	}
	
//	if (ret == 1)
//	{
//		smilSeek (1, ncx);
//		printf ("\n\n\n");
//		printf ("Smilfil: %s\n\n", smilPos->anchor);
//		struct Node *tmp = nodeHead->next;
//		while (tmp != nodeTail)
//		{
//			printf ("Mp3: %s Start: %s Slutt: %s\n", tmp->audiofilename, tmp->audioStartPos, tmp->audioStopPos);
//			tmp = tmp->next;	
//		}
//		printf ("\n\n\n");
//	}
	
	
	if (ret == 1)
	{		
		printf("\n\n");
		int playing = 1;
		
		while(playing)
		{	
			playing = seek (2, ncx);
			if (!playing) break;
			if(nodePos->textPassage != NULL)
				printf("%s\n", nodePos->textPassage);
			if(nodePos->audiofilename != NULL)
			{
				char tmp[STRLEN];
				strcpy(tmp, path);
				strcat(tmp, "/");
				strcat(tmp, nodePos->audiofilename);
				//printf ("\nPlaying: %s From: %s To: %s", nodePos->audiofilename, nodePos->audioStartPos, nodePos->audioStopPos);
				play(tmp, nodePos->audioStartPos, nodePos->audioStopPos);	
			}
			// if we don't got any audiosource the text must be delayed for an appropriate time, this
			// is done by sleeping in the length og the string divided by 40.
			else
			{
				sleep(strlen(nodePos->textPassage)/40);
			}			
		}
		report ("\n\nEnd of book\n", REP_MESSAGE);
	}
	
	parseTerminate();
	//mp3_close();
	return 0;
}


/**
 * Start parsing a NCC or a NCX file.
 * 
 * @param filenam containing the path/filename to the ncc/ncx file
 * 
 * @returns 1 in case of success and -1 in case of error
 */
int parse(char* filename)
{
//	char tmp[STRLEN];
	char buffer[STRLEN];
	int i, j, n, ret;
	j = n = ret = 0;
	i = strlen(filename);
	
	// checks wheter the filename ends with .ncx
	if(filename[i-1] == 'x' && filename[i-2] == 'c' && filename[i-3] == 'n' && filename[i-4] == '.')
	{	
		// parse NCX, must extract the path
		ncx = 1;
		// find the path
		for(i = strlen(filename)-1; i > 0; i--)
		{
			if(filename[i] == '/')
				break;
		}
		// stores the path
		for (j = 0; j <= i; j++)
		{
			buffer[n++] = filename[j];
		}
		buffer[n] = '\0';
		path = (char*) malloc (sizeof (buffer)+1);
		strcpy (path, buffer);
		ret = parseNCX(filename);	
	}
	// 2.02 Daisy book must be ../ncc.html
	else if(filename[i-1] == 'l' && filename[i-2] == 'm' && filename[i-3] == 't' && filename[i-4] == 'h'
	&&	filename[i-5] == '.' && filename[i-6] == 'c' && filename[i-7] == 'c' && filename[i-8] == 'n' && filename[i-9] == '/')
	{
		// we got a 2.02 Daisy book -> parseNCC
		ncx = 0;
		
		// stores the path
		for (j = 0; j <= strlen(filename)-9; j++)
		{
			buffer[n++] = filename[j];
		}
		buffer[n] = '\0';
		path = (char*) malloc (sizeof (buffer)+1);
		strcpy (path, buffer);
		ret = parseNCC(filename);
	}
	else
	{
		printf ("\n\nSorry, you must enter a valid path for the Daisybook (../ncc.html | ../*.ncx)\n\n");
		return -1;
	}
	return ret;
}


/**
 * Listens for CTRL-C and does it's best to free resources in a Good Way
 */
static void sigproc()
{
	signal(SIGINT, sigproc);	/** reset the signal */
	report ("Recieved ctrl-c. Cleaning up and exiting.\n", REP_MESSAGE);
	parseTerminate();
	//mp3_close();	/** tells mp3 to close. it will in turn free libao */
	exit (0);
}
